function [x0,y,Omega,M,Lambda]=Generate_Problem(d,p,m,k,noiselevel)

% Building an analysis problem, which includes the ingredients: 
%   - Omega - the analysis operator of size p*d
%   - M is anunderdetermined measurement matrix of size m*d (m<d)
%   - x0 is a vector of length d that satisfies ||Omega*x0||=p-k
%   - Lambda is the true location of these k zeros in Omega*x0
%   - a measurement vector y0=Mx0 is computed
%   - noise contaminated measurement vector y is obtained by
%     y = y0 + n where n is an additive gaussian noise with norm(n,2)/norm(y0,2) = noiselevel

if nargin==0 % default values
    d=100;
    p=110;
    m=50; 
    k=85; 
end;

Omega=Generate_Analysis_Operator(d, p);

Lambda=randperm(p); 
Lambda=sort(Lambda(1:k)); 

if 1 % general M matrix
    M=randn(m,d);
else % inpainting
    M=eye(d);
    pos=randperm(d);
    pos=pos(1:m);
    M=M(sort(pos),:);
end

% The signal is drawn at random from the null-space defined by the rows 
% of the matreix Omega(Lambda,:)
[U,D,V]=svd(Omega(Lambda,:));
NullSpace=V(:,k+1:d);
x0=NullSpace*randn(d-k,1);
y=M*x0;

t_norm = norm(y,2);
n = randn(m, 1);
y = y + noiselevel * t_norm * n / norm(n, 2);

return

